/*******************************************************************************
* File Name: Divider.c  
* Version 2.10
*
* Description:
*  The PWM User Module consist of an 8 or 16-bit counter with two 8 or 16-bit
*  comparitors. Each instance of this user module is capable of generating
*  two PWM outputs with the same period. The pulse width is selectable between
*  1 and 255/65535. The period is selectable between 2 and 255/65536 clocks. 
*  The compare value output may be configured to be active when the present 
*  counter is less than or less than/equal to the compare value.
*  A terminal count output is also provided. It generates a pulse one clock
*  width wide when the counter is equal to zero.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "Divider.h"

uint8 Divider_initVar = 0u;

/*******************************************************************************
* Function Name: Divider_Start
********************************************************************************
*
* Summary:
*  The start function initializes the pwm with the default values, the 
*  enables the counter to begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  Divider_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(Divider_initVar == 0u)
    {
        Divider_Init();
        Divider_initVar = 1u;
    }
    Divider_Enable();

}


/*******************************************************************************
* Function Name: Divider_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  Divider_Start().
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_Init(void) 
{
    #if (Divider_UsingFixedFunction || Divider_UseControl)
        uint8 ctrl;
    #endif
    
    #if(!Divider_UsingFixedFunction) 
        #if(Divider_UseStatus)
            /* Interrupt State Backup for Critical Region*/
            uint8 Divider_interruptState;
        #endif
    #endif
    
   #if (Divider_UsingFixedFunction)
        /* You are allowed to write the compare value (FF only) */
        Divider_CONTROL |= Divider_CFG0_MODE;
        #if (Divider_DeadBand2_4)
            Divider_CONTROL |= Divider_CFG0_DB;
        #endif
                
        /* Set the default Compare Mode */
        #if(Divider_PSOC3_ES2 || Divider_PSOC5_ES1)
                ctrl = Divider_CONTROL2 & ~Divider_CTRL_CMPMODE1_MASK;
                Divider_CONTROL2 = ctrl | Divider_DEFAULT_COMPARE1_MODE;
        #endif
        #if(Divider_PSOC3_ES3 || Divider_PSOC5_ES2)
                ctrl = Divider_CONTROL3 & ~Divider_CTRL_CMPMODE1_MASK;
                Divider_CONTROL3 = ctrl | Divider_DEFAULT_COMPARE1_MODE;
        #endif
        
         /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
        Divider_RT1 &= ~Divider_RT1_MASK;
        Divider_RT1 |= Divider_SYNC;     
                
        /*Enable DSI Sync all all inputs of the PWM*/
        Divider_RT1 &= ~(Divider_SYNCDSI_MASK);
        Divider_RT1 |= Divider_SYNCDSI_EN;
       
    #elif (Divider_UseControl)
        /* Set the default compare mode defined in the parameter */
        ctrl = Divider_CONTROL & ~Divider_CTRL_CMPMODE2_MASK & ~Divider_CTRL_CMPMODE1_MASK;
        Divider_CONTROL = ctrl | Divider_DEFAULT_COMPARE2_MODE | Divider_DEFAULT_COMPARE1_MODE;
    #endif 
        
    #if (!Divider_UsingFixedFunction)
        #if (Divider_Resolution == 8)
            /* Set FIFO 0 to 1 byte register for period*/
            Divider_AUX_CONTROLDP0 |= (Divider_AUX_CTRL_FIFO0_CLR);
        #else /* (Divider_Resolution == 16)*/
            /* Set FIFO 0 to 1 byte register for period */
            Divider_AUX_CONTROLDP0 |= (Divider_AUX_CTRL_FIFO0_CLR);
            Divider_AUX_CONTROLDP1 |= (Divider_AUX_CTRL_FIFO0_CLR);
        #endif
    #endif
        
    Divider_WritePeriod(Divider_INIT_PERIOD_VALUE);
    Divider_WriteCounter(Divider_INIT_PERIOD_VALUE);
        
        #if (Divider_UseOneCompareMode)
            Divider_WriteCompare(Divider_INIT_COMPARE_VALUE1);
        #else
            Divider_WriteCompare1(Divider_INIT_COMPARE_VALUE1);
            Divider_WriteCompare2(Divider_INIT_COMPARE_VALUE2);
        #endif
        
        #if (Divider_KillModeMinTime)
            Divider_WriteKillTime(Divider_MinimumKillTime);
        #endif
        
        #if (Divider_DeadBandUsed)
            Divider_WriteDeadTime(Divider_INIT_DEAD_TIME);
        #endif

    #if (Divider_UseStatus || Divider_UsingFixedFunction)
        Divider_SetInterruptMode(Divider_INIT_INTERRUPTS_MODE);
    #endif
        
    #if (Divider_UsingFixedFunction)
        /* Globally Enable the Fixed Function Block chosen */
        Divider_GLOBAL_ENABLE |= Divider_BLOCK_EN_MASK;
        /* Set the Interrupt source to come from the status register */
        Divider_CONTROL2 |= Divider_CTRL2_IRQ_SEL;
    #else
        #if(Divider_UseStatus)
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            Divider_interruptState = CyEnterCriticalSection();
            /* Use the interrupt output of the status register for IRQ output */
            Divider_STATUS_AUX_CTRL |= Divider_STATUS_ACTL_INT_EN_MASK;
            
             /* Exit Critical Region*/
            CyExitCriticalSection(Divider_interruptState);
            
            /* Clear the FIFO to enable the Divider_STATUS_FIFOFULL
                   bit to be set on FIFO full. */
            Divider_ClearFIFO();
        #endif
    #endif
}


/*******************************************************************************
* Function Name: Divider_Enable
********************************************************************************
*
* Summary: 
*  Enables the PWM block operation
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Side Effects: 
*  This works only if software enable mode is chosen
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (Divider_UsingFixedFunction)
        Divider_GLOBAL_ENABLE |= Divider_BLOCK_EN_MASK;
        Divider_GLOBAL_STBY_ENABLE |= Divider_BLOCK_STBY_EN_MASK;
    #endif 
    
    /* Enable the PWM from the control register  */
    #if (Divider_UseControl || Divider_UsingFixedFunction)
        Divider_CONTROL |= Divider_CTRL_ENABLE;
    #endif
}


/*******************************************************************************
* Function Name: Divider_Stop
********************************************************************************
*
* Summary:
*  The stop function halts the PWM, but does not change any modes or disable
*  interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects:
*  If the Enable mode is set to Hardware only then this function
*  has no effect on the operation of the PWM
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_Stop(void) 
{
    #if (Divider_UseControl || Divider_UsingFixedFunction)
        Divider_CONTROL &= ~Divider_CTRL_ENABLE;
    #endif
    
    /* Globally disable the Fixed Function Block chosen */
    #if (Divider_UsingFixedFunction)
        Divider_GLOBAL_ENABLE &= ~Divider_BLOCK_EN_MASK;
        Divider_GLOBAL_STBY_ENABLE &= ~Divider_BLOCK_STBY_EN_MASK;
    #endif
}


#if (Divider_UseOneCompareMode)
#if (Divider_CompareMode1SW)
/*******************************************************************************
* Function Name: Divider_SetCompareMode
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm output when in Dither mode,
*  Center Align Mode or One Output Mode.
*
* Parameters:
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return:
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_SetCompareMode(uint8 comparemode) 
{
    #if(Divider_UsingFixedFunction)
            #if(Divider_PSOC3_ES2 || Divider_PSOC5_ES1)
                        uint8 comparemodemasked = (comparemode << Divider_CTRL_CMPMODE1_SHIFT);
                        Divider_CONTROL2 &= ~Divider_CTRL_CMPMODE1_MASK; /*Clear Existing Data */
                        Divider_CONTROL2 |= comparemodemasked;  
                #endif
                
            #if(Divider_PSOC3_ES3 || Divider_PSOC5_ES2)
                    uint8 comparemodemasked = (comparemode << Divider_CTRL_CMPMODE1_SHIFT);
            Divider_CONTROL3 &= ~Divider_CTRL_CMPMODE1_MASK; /*Clear Existing Data */
            Divider_CONTROL3 |= comparemodemasked;     
                #endif
                
    #elif (Divider_UseControl)
        uint8 comparemode1masked = (comparemode << Divider_CTRL_CMPMODE1_SHIFT) & Divider_CTRL_CMPMODE1_MASK;
        uint8 comparemode2masked = (comparemode << Divider_CTRL_CMPMODE2_SHIFT) & Divider_CTRL_CMPMODE2_MASK;
        Divider_CONTROL &= ~(Divider_CTRL_CMPMODE1_MASK | Divider_CTRL_CMPMODE2_MASK); /*Clear existing mode */
        Divider_CONTROL |= (comparemode1masked | comparemode2masked);
        
    #else
        uint8 temp = comparemode;
    #endif
}
#endif /* Divider_CompareMode1SW */

#else /* UseOneCompareMode */


#if (Divider_CompareMode1SW)
/*******************************************************************************
* Function Name: Divider_SetCompareMode1
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm or pwm1 output
*
* Parameters:  
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_SetCompareMode1(uint8 comparemode) 
{
    uint8 comparemodemasked = (comparemode << Divider_CTRL_CMPMODE1_SHIFT) & Divider_CTRL_CMPMODE1_MASK;
    #if(Divider_UsingFixedFunction)
            #if(Divider_PSOC3_ES2 || Divider_PSOC5_ES1)
                        Divider_CONTROL2 &= Divider_CTRL_CMPMODE1_MASK; /*Clear existing mode */
                        Divider_CONTROL2 |= comparemodemasked; 
            #endif
                
                #if(Divider_PSOC3_ES3 || Divider_PSOC5_ES2)
                    Divider_CONTROL3 &= Divider_CTRL_CMPMODE1_MASK; /*Clear existing mode */
                        Divider_CONTROL3 |= comparemodemasked; 
            #endif
                
    #elif (Divider_UseControl)
        Divider_CONTROL &= Divider_CTRL_CMPMODE1_MASK; /*Clear existing mode */
        Divider_CONTROL |= comparemodemasked;
    #endif    
}
#endif /* Divider_CompareMode1SW */


#if (Divider_CompareMode2SW)
/*******************************************************************************
* Function Name: Divider_SetCompareMode2
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm or pwm2 output
*
* Parameters:  
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_SetCompareMode2(uint8 comparemode) 
{
    #if(Divider_UsingFixedFunction)
        /* Do Nothing because there is no second Compare Mode Register in FF block*/ 
    #elif (Divider_UseControl)
    uint8 comparemodemasked = (comparemode << Divider_CTRL_CMPMODE2_SHIFT) & Divider_CTRL_CMPMODE2_MASK;
    Divider_CONTROL &= Divider_CTRL_CMPMODE2_MASK; /*Clear existing mode */
    Divider_CONTROL |= comparemodemasked;
    #endif    
}
#endif /*Divider_CompareMode2SW */
#endif /* UseOneCompareMode */


/*******************************************************************************
* Function Name: Divider_WriteCounter
********************************************************************************
* 
* Summary:
*  This function is used to change the counter value.
*
* Parameters:  
*  counter:  This value may be between 1 and (2^Resolution)-1.   
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_WriteCounter(uint16 counter) 
{
    #if(Divider_UsingFixedFunction)
        CY_SET_REG16(Divider_COUNTER_LSB_PTR, (uint16)counter);
    #else
        CY_SET_REG16(Divider_COUNTER_LSB_PTR, counter);
    #endif
}


#if (!Divider_UsingFixedFunction)
/*******************************************************************************
* Function Name: Divider_ReadCounter
********************************************************************************
* 
* Summary:
*  This function returns the current value of the counter.  It doesn't matter
*  if the counter is enabled or running.
*
* Parameters:  
*  void  
*
* Return: 
*  The current value of the counter.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint16 Divider_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(Divider_COUNTERCAP_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG16(Divider_CAPTURE_LSB_PTR));
}


#if (Divider_UseStatus)
/*******************************************************************************
* Function Name: Divider_ClearFIFO
********************************************************************************
* 
* Summary:
*  This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_ClearFIFO(void) 
{
    while(Divider_ReadStatusRegister() & Divider_STATUS_FIFONEMPTY)
        Divider_ReadCapture();
}
#endif /* Divider_UseStatus */
#endif /* !Divider_UsingFixedFunction */


/*******************************************************************************
* Function Name: Divider_WritePeriod
********************************************************************************
* 
* Summary:
*  This function is used to change the period of the counter.  The new period 
*  will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period:  Period value. May be between 1 and (2^Resolution)-1.  A value of 0 
*           will result in the counter remaining at zero.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_WritePeriod(uint16 period) 
{
    #if(Divider_UsingFixedFunction)
        CY_SET_REG16(Divider_PERIOD_LSB_PTR, (uint16)period);
    #else
        CY_SET_REG16(Divider_PERIOD_LSB_PTR, period);
    #endif
}


#if (Divider_UseOneCompareMode)
/*******************************************************************************
* Function Name: Divider_WriteCompare
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare1 value when the PWM is in Dither
*  mode. The compare output will reflect the new value on the next UDB clock. 
*  The compare output will be driven high when the present counter value is 
*  compared to the compare value based on the compare mode defined in 
*  Dither Mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Side Effects:
*  This function is only available if the PWM mode parameter is set to
*  Dither Mode, Center Aligned Mode or One Output Mode
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_WriteCompare(uint16 compare) 
{
   CY_SET_REG16(Divider_COMPARE1_LSB_PTR, compare);
   #if (Divider_PWMMode == Divider__B_PWM__DITHER)
        CY_SET_REG16(Divider_COMPARE2_LSB_PTR, compare+1);
   #endif
}


#else


/*******************************************************************************
* Function Name: Divider_WriteCompare1
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare1 value.  The compare output will 
*  reflect the new value on the next UDB clock.  The compare output will be 
*  driven high when the present counter value is less than or less than or 
*  equal to the compare register, depending on the mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_WriteCompare1(uint16 compare) 
{
    #if(Divider_UsingFixedFunction)
        CY_SET_REG16(Divider_COMPARE1_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG16(Divider_COMPARE1_LSB_PTR, compare);
    #endif
}


/*******************************************************************************
* Function Name: Divider_WriteCompare2
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare value, for compare1 output.  
*  The compare output will reflect the new value on the next UDB clock.  
*  The compare output will be driven high when the present counter value is 
*  less than or less than or equal to the compare register, depending on the 
*  mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_WriteCompare2(uint16 compare) 
{
    #if(Divider_UsingFixedFunction)
        CY_SET_REG16(Divider_COMPARE2_LSB_PTR, compare);
    #else
        CY_SET_REG16(Divider_COMPARE2_LSB_PTR, compare);
    #endif
}
#endif /* UseOneCompareMode */


#if (Divider_DeadBandUsed)
/*******************************************************************************
* Function Name: Divider_WriteDeadTime
********************************************************************************
* 
* Summary:
*  This function writes the dead-band counts to the corresponding register
*
* Parameters:  
*  deadtime:  Number of counts for dead time 
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Divider_WriteDeadTime(uint8 deadtime) 
{
    /* If using the Dead Band 1-255 mode then just write the register */
    #if(!Divider_DeadBand2_4)
        CY_SET_REG8(Divider_DEADBAND_COUNT_PTR, deadtime);
    #else
        /* Otherwise the data has to be masked and offset */        
        /* Clear existing data */
        Divider_DEADBAND_COUNT &= ~Divider_DEADBAND_COUNT_MASK; 
            /* Set new dead time */
        Divider_DEADBAND_COUNT |= (deadtime << Divider_DEADBAND_COUNT_SHIFT) & Divider_DEADBAND_COUNT_MASK; 
    #endif
}


/*******************************************************************************
* Function Name: Divider_ReadDeadTime
********************************************************************************
* 
* Summary:
*  This function reads the dead-band counts from the corresponding register
*
* Parameters:  
*  void
*
* Return: 
*  Dead Band Counts
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint8 Divider_ReadDeadTime(void) 
{
    /* If using the Dead Band 1-255 mode then just read the register */
    #if(!Divider_DeadBand2_4)
        return (CY_GET_REG8(Divider_DEADBAND_COUNT_PTR));
    #else
        /* Otherwise the data has to be masked and offset */
        return ((Divider_DEADBAND_COUNT & Divider_DEADBAND_COUNT_MASK) >> Divider_DEADBAND_COUNT_SHIFT);
    #endif
}
#endif /* DeadBandUsed */

