/*******************************************************************************
* File Name: LCD.h
* Version 1.20
*
* Description:
*  This header file contains registers and constants associated with the 
*  Character LCD component.
*
* Note:
* 
********************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/



#if !defined(LCD_H) /* Character LCD Header File */
#define LCD_H

#include "cytypes.h"
#include "cyfitter.h"
#include "device.h"

/***************************************
*        Function Prototypes
***************************************/

void LCD_Start(void);
void LCD_Stop(void);
void LCD_WriteControl(uint8 cByte);
void LCD_WriteData(uint8 dByte);
void LCD_PrintString(char8 *);
void LCD_Position(uint8 row, uint8 column);
void LCD_PutChar(char8 character);
void LCD_IsReady(void);
void LCD_WrDatNib(uint8 nibble);
void LCD_WrCntrlNib(uint8 nibble);
void LCD_DelayUS(uint8 blocks);

/* ASCII Conversion Routines */                        
void LCD_PrintHexUint8(uint8 value);      
void LCD_PrintHexUint16(uint16 value);    
void LCD_PrintDecUint16(uint16 value);   
#define LCD_PrintNumber(x)   LCD_PrintDecUint16(x)
#define LCD_PrintInt8(x)     LCD_PrintHexUint8(x)
#define LCD_PrintInt16(x)    LCD_PrintHexUint16(x)

/* Clear Macro */
#define LCD_ClearDisplay() LCD_WriteControl(LCD_CLEAR_DISPLAY)

/* Off Macro */
#define LCD_DisplayOff() LCD_WriteControl(LCD_DISPLAY_CURSOR_OFF)

/* On Macro */
#define LCD_DisplayOn() LCD_WriteControl(LCD_DISPLAY_ON_CURSOR_OFF)


/***************************************
*           API Constants        
***************************************/

/* Full Byte Commands Sent as Two Nibbles */
#define LCD_DISPLAY_8_BIT_INIT      0x03u
#define LCD_DISPLAY_4_BIT_INIT      0x02u
#define LCD_DISPLAY_CURSOR_OFF      0x08u
#define LCD_CLEAR_DISPLAY           0x01u
#define LCD_CURSOR_AUTO_INCR_ON     0x06u
#define LCD_DISPLAY_CURSOR_ON       0x0Eu
#define LCD_DISPLAY_2_LINES_5x10    0x2Cu
#define LCD_DISPLAY_ON_CURSOR_OFF   0x0Cu

#define LCD_RESET_CURSOR_POSITION   0x03u
#define LCD_CURSOR_WINK             0x0Du
#define LCD_CURSOR_BLINK            0x0Fu
#define LCD_CURSOR_SH_LEFT          0x10u
#define LCD_CURSOR_SH_RIGHT         0x14u
#define LCD_CURSOR_HOME             0x02u
#define LCD_CURSOR_LEFT             0x04u
#define LCD_CURSOR_RIGHT            0x06u
/* Point to Character Generator Ram 0 */             
#define LCD_CGRAM_0                 0x40u
/* Point to Display Data Ram 0 */
#define LCD_DDRAM_0                 0x80u

/* LCD Characteristics */
#define LCD_CHARACTER_WIDTH         0x05u
#define LCD_CHARACTER_HEIGHT        0x08u

/* Nibble Offset and Mask */
#define LCD_NIBBLE_SHIFT            0x04u
#define LCD_NIBBLE_MASK             0x0Fu

/* LCD Module Address Constants */
#define LCD_ROW_0_START             0x80u
#define LCD_ROW_1_START             0xC0u
#define LCD_ROW_2_START             0x94u
#define LCD_ROW_3_START             0xD4u

/* Bargraph and Custom Character Constants */
#define LCD_NONE   0x00u   /* No Custom Fonts      */
#define LCD_HBG    0x01u   /* Horizontal Bar Graph */
#define LCD_VBG    0x02u   /* Vertical Bar Graph   */
#define LCD_UD     0x03u   /* User Defined Fonts   */




/***************************************
*             Registers        
***************************************/

/* Port Register Definitions */
#define LCD_PORT_DR        (*(reg8 *) LCD_LCDPort__DR)   /* Data Output Register */
#define LCD_PORT_PS        (*(reg8 *) LCD_LCDPort__PS)   /* Pin State Register */
#define LCD_PORT_DM0       (*(reg8 *) LCD_LCDPort__DM0)  /* Port Drive Mode 0 */
#define LCD_PORT_DM1       (*(reg8 *) LCD_LCDPort__DM1)  /* Port Drive Mode 1 */
#define LCD_PORT_DM2       (*(reg8 *) LCD_LCDPort__DM2)  /* Port Drive Mode 2 */


/***************************************
*       Register Constants        
***************************************/

/* SHIFT must be 1 or 0 */
#define LCD_PORT_SHIFT              LCD_LCDPort__SHIFT
#define LCD_PORT_MASK               LCD_LCDPort__MASK

/**Drive Mode register values for High Z */
#define LCD_HIGH_Z_DM0             0xFFu
#define LCD_HIGH_Z_DM1             0x00u
#define LCD_HIGH_Z_DM2             0x00u

/* Drive Mode register values for Strong */
#define LCD_STRONG_DM0             0x00u
#define LCD_STRONG_DM1             0xFFu
#define LCD_STRONG_DM2             0xFFu

/* Pin Masks */
#define LCD_RS                      (0x20u << LCD_LCDPort__SHIFT)
#define LCD_RW                      (0x40u << LCD_LCDPort__SHIFT)
#define LCD_E                       (0x10u << LCD_LCDPort__SHIFT)
#define LCD_READY_BIT               (0x08u << LCD_LCDPort__SHIFT)
#define LCD_DATA_MASK               (0x0Fu << LCD_LCDPort__SHIFT)

#endif /* End Character LCD Header File */


/* [] END OF FILE */
